unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls;

const
  SpeichenCnt = 5;
  minPhi = pi / 36 / SpeichenCnt;
  maxPhi = 2 * pi / SpeichenCnt;
  Wachstum = exp(1 / 5 * ln(2));// 2^(1/n)

type

  { TRadPanel }
  TRadPanel = class(TPanel)
  private
    FTmpBmp,
    FTmpBmpOrg,
    FTmpKolben: TBitMap;

    FSpeichCol: array[0..SpeichenCnt - 1] of TColor;
    FWinkel: double;
    FpicFertig: boolean;
    procedure SetWinkel(AValue: double);
  public
    property Winkel: double read FWinkel write SetWinkel;
    constructor Create(TheOwner: TComponent); override;
    procedure NewPic(Sender: TObject);
    procedure RadPanelPaint(Sender: TObject);
    procedure RadPanelResize(Sender: TObject; w, h: integer);
  end;


  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    ScrollBar1: TScrollBar;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ScrollBar1Change(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    Schritt: double;
  public
    RadPanel: TRadPanel;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TRadPanel }

procedure TRadPanel.SetWinkel(AValue: double);
begin
  if FWinkel <>  AValue then
    FWinkel := AValue;
end;

constructor TRadPanel.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FSpeichCol[0] := clLime;
  FSpeichCol[1] := clgray;
  FSpeichCol[2] := clyellow;
  FSpeichCol[3] := clblue;
  FSpeichCol[4] := clred;
  OnPaint := @NewPic;
end;

procedure TRadPanel.NewPic(Sender: TObject);
var
  RadiusRad, dR, i: integer;
  l: integer;
  p: integer;
  r, sWi, cWi: single;
  radz, z: TPoint;
  tmpCol: tColor;
begin
  FpicFertig := False;
  FtmpBmp.Canvas.Draw(0,0,FtmpBmpOrg);

  radz.x := FtmpBmp.Width div 5;
  radz.y := FtmpBmp.Height div 2;
  RadiusRad := radz.y div 3 * 2;
  r := RadiusRad / 2;
  dR := radz.y div 16;
  l := RadiusRad*3 div 2 +dR;
  RadiusRad := RadiusRad - dR div 2;

  tmpCol := Canvas.Pen.Color;
  for i := 0 to SpeichenCnt - 1 do
  begin
    FtmpBmp.Canvas.Pen.Color := fSpeichCol[i];
    sWi := sin(Pi * 2 / SpeichenCnt * i + FWinkel);
    cWi := cos(Pi * 2 / SpeichenCnt * i + FWinkel);
    FtmpBmp.Canvas.Line(radz.x + Round(cWi * dR), radz.y + Round(sWi * dR),
      radz.x + round(cWi * RadiusRad),
      radz.y + round(sWi * RadiusRad));
  end;
  FtmpBmp.Canvas.Pen.Color := tmpCol;

  dr := dr div 2;
  FtmpBmp.Canvas.EllipseC(radz.x, radz.y, dR,dR);

  sWi := sin(FWinkel);
  cWi := cos(FWinkel);
  z.y := round(sWi * r);
  z.x := round(cWi * r);

  FtmpBmp.Canvas.EllipseC(radz.x + z.x, radz.y + z.y, 10, 10);

//  p := round(-(l  - (r * cWi + l * sqrt(1 - sqr(r / l) * sqr(sWi)))));
  p := round(r*(sqrt(sqr(l/r)-sqr(sWi))+cWi)-l);
  FtmpBmp.Canvas.Draw(radz.x + p + l-dr,radz.y-FTmpKolben.height div 2,FTmpKolben);
  FtmpBmp.Canvas.Line(radz.x + z.x, radz.y + z.y, radz.x + p + l, radz.y);
  //Testet ob p richtig berechnet, Kreis mit Radius Pleuellaenge == l
//  FtmpBmp.Canvas.ellipseC(radz.x + z.x, radz.y + z.y,l,l);
  FpicFertig := True;
end;

procedure TRadPanel.RadPanelPaint(Sender: TObject);
begin
  if FpicFertig then
    self.Canvas.Draw(0, 0, FtmpBmp);
end;

procedure TRadPanel.RadPanelResize(Sender: TObject; w, h: integer);
var
  x,y,RadiusRad,dR : integer;
begin
  with sender do
    if Assigned(FtmpBmp) then
    begin
      FtmpBmp.Clear;
      FtmpBmp.setsize(W, H);
      FtmpBmp.Canvas.brush.Color := clwindow;
      FtmpBmp.Canvas.brush.Style := bsSolid;
      FtmpBmp.Canvas.pen.Color := clBlack;

      FtmpBmpOrg.Clear;
      FtmpBmpOrg.setsize(W, H);
      FtmpBmpOrg.Canvas.brush.Color := clwindow;
      FtmpBmpOrg.Canvas.brush.Style := bsSolid;
      FtmpBmpOrg.Canvas.pen.Color := clBlack;
      FtmpBmpOrg.Canvas.Fillrect(clientrect);
      FtmpBmp.Canvas.Brush.Style := bsclear;

      x := w div 5;
      y := h div 2;
      RadiusRad := y div 3 * 2;

      dR := y div 16;
      FtmpBmp.Canvas.Pen.Width := dR;
      FtmpBmpOrg.Canvas.Pen.Width := dR;

      FtmpBmpOrg.Canvas.EllipseC(x, y, RadiusRad, RadiusRad);

//      x := w div 5;
//      y := x;
      with FTmpKolben do
      Begin
        Clear;
        setsize(x,y);
        with canvas do
        Begin
          brush.Color := clwindow;
          brush.Style := bsSolid;
          pen.Color := clBlack;
          Fillrect(clientrect);
          Pen.Width := dR;
          dR := dr div 2;
          Line(dr, y div 2, x div 2,y div 2);
          Rectangle( x div 2+dr, dr,x-dr,y-dr);
          brush.Color := clblack;
          EllipseC(10+dr,y div 2, 10, 10);
        end;
      end;
    end;

end;

{ TForm1 }

procedure TForm1.FormShow(Sender: TObject);
begin
  Schritt := minPhi;
  RadPanel := TRadPanel.Create(Self);
  RadPanel.Align := alClient;
  RadPanel.Parent := Self;
  DoubleBuffered := true;

  Button1.Parent := RadPanel;
  Button2.Parent := RadPanel;
  Scrollbar1.Parent := RadPanel;
  Button3.Parent := RadPanel;
  Button3.Enabled := true;

  Scrollbar1.Position:= Timer1.Interval;
  Button3.Caption:=IntToStr(Timer1.interval);

  RadPanel.FTmpBmp := TBitmap.Create;
  RadPanel.FTmpBmpOrg := TBitmap.Create;
  RadPanel.FTmpKolben := TBitmap.Create;
  RadPanel.RadPanelResize(self, self.Width, self.Height);
end;

procedure TForm1.ScrollBar1Change(Sender: TObject);
begin
  Timer1.Enabled := False;
  Timer1.Interval := Scrollbar1.Position;
  Button3.Caption:=IntToStr(Timer1.interval);
  application.ProcessMessages;
  Timer1.Enabled := True;
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
  Schritt /= Wachstum;
  if Schritt < minPhi then
    Schritt := minPhi;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  Schritt *= Wachstum;
  if Schritt > maxPhi then
    Schritt := maxPhi;
end;


procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  RadPanel.FTmpKolben.free;
  RadPanel.FTmpBmpOrg.free;
  RadPanel.FTmpBmp.Free;
  RadPanel.Free;
end;


procedure TForm1.FormResize(Sender: TObject);
// Wird schon vor show aufgerufen
begin
  IF RadPanel <> NIL then
    RadPanel.RadPanelResize(RadPanel, self.Width, self.Height);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  RadPanel.Winkel := RadPanel.Winkel + Schritt;
  while RadPanel.Winkel > 2 * Pi do
    RadPanel.Winkel := RadPanel.Winkel - 2 * Pi;
  Repaint;
  RadPanel.RadPanelPaint(self);
  Caption := Format('Schritt: %5.3f Winkel: %6.3f', [Schritt, RadPanel.Winkel]);
end;

end.
